//	PhilComponent.cpp - The Phil file manager/browser Component.
//	----------------------------------------------------------------------------
//	This file is part of 'Phil', a very simple file manager/browser for the
//	JUCE library.
//	Copyright (C) 2005-2006  Niall Moody
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//	----------------------------------------------------------------------------

//#define PHIL_LIGHT

#include <juce.h>
#include "PhilComponent.h"
#include "ListFileView.h"
#include "PhilChooser.h"
#ifndef PHIL_LIGHT
#include "FilePropertiesDialog.h"
#endif

//------------------------------------------------------------------------------
Phil::Phil():
Component(T("Phil")),
ChangeBroadcaster(),
fileView(0),
currentDirectory(SystemStats::getUserHomeDirectory()),
multipleFilesSelected(true),
isCut(false),
currentVisibleExtensions(-1),
displayContextMenu(true),
allowExternalPrograms(true),
allowDragAndDrop(true),
fileViewBoundaryColour(Colours::lightgrey),
fileViewBackgroundColour(Colours::white),
fileViewTextColour(Colours::grey),
fileViewHighlightColour(Colours::lightblue),
toolbarComponent(0),
toolbarHeight(32),
sidebarComponent(0),
sidebarWidth(128),
statusbarComponent(0),
statusbarHeight(24)
{
	addAndMakeVisible(view = new Viewport());
	view->setScrollBarsShown(false, true);
	view->setViewedComponent(fileView = new ListFileView(this));

	//setupShortcuts();
	appCommands.registerAllCommandsForTarget(this);
	appCommands.getKeyMappings()->resetToDefaultMappings();
	//addKeyListener(appCommands.getKeyMappings());
}

//------------------------------------------------------------------------------
Phil::~Phil()
{
	deleteAllChildren();
}

//------------------------------------------------------------------------------
void Phil::paint(Graphics& g)
{
	//Draw rounded rectangle marking the bounds of the file view.
	g.setColour(fileViewBoundaryColour);
	g.drawRoundedRectangle(fileViewBounds.getX()+2,
						   fileViewBounds.getY()+2,
						   fileViewBounds.getWidth()-4,
						   fileViewBounds.getHeight()-4,
						   5,
						   4);

	//Colour background of the file view rounded rectangle.
	g.setColour(fileViewBackgroundColour);
	g.fillRoundedRectangle(fileViewBounds.getX()+2,
						   fileViewBounds.getY()+2,
						   fileViewBounds.getWidth()-4,
						   fileViewBounds.getHeight()-4,
						   5);
}

//------------------------------------------------------------------------------
void Phil::resized()
{
	int tempx, tempy;
	int numFiles;
	OwnedArray<File> children;

	if(toolbarComponent)
		toolbarComponent->setBounds(0, 0, getWidth(), toolbarHeight);
	if(sidebarComponent)
	{
		if((toolbarComponent)&&(statusbarComponent))
		{
			sidebarComponent->setBounds(0,
										toolbarHeight,
										sidebarWidth,
										(getHeight()-(toolbarHeight+statusbarHeight)));
		}
		else if(toolbarComponent)
		{
			sidebarComponent->setBounds(0,
										toolbarHeight,
										sidebarWidth,
										(getHeight()-toolbarHeight));
		}
		else if(statusbarComponent)
		{
			sidebarComponent->setBounds(0,
										toolbarHeight,
										sidebarWidth,
										(getHeight()-statusbarHeight));
		}
		else
		{
			sidebarComponent->setBounds(0,
										toolbarHeight,
										sidebarWidth,
										getHeight());
		}
	}
	if(statusbarComponent)
	{
		statusbarComponent->setBounds(0,
									  (getHeight()-statusbarHeight),
									  getWidth(),
									  statusbarHeight);
	}

	if(sidebarComponent)
		tempx = sidebarWidth;
	else
		tempx = 0;
	if(toolbarComponent)
		tempy = toolbarHeight;
	else
		tempy = 0;
	fileViewBounds.setPosition(tempx, tempy);

	if(sidebarComponent)
		tempx = getWidth() - sidebarWidth;
	else
		tempx = getWidth();
	if(toolbarComponent)
		tempy = getHeight() - toolbarHeight;
	else
		tempy = getHeight();
	if(statusbarComponent)
		tempy -= statusbarHeight;
	fileViewBounds.setSize(tempx, tempy);

	view->setBounds((fileViewBounds.getX()+7),
					(fileViewBounds.getY()+4),
					(fileViewBounds.getWidth()-14),
					(fileViewBounds.getHeight()-8));

	findChildFiles(children, true);
	numFiles = children.size();
	children.clear();
	findChildFiles(children, false);
	numFiles += children.size();
	fileView->updateSize((fileViewBounds.getWidth()-22),
						 (fileViewBounds.getHeight()-22),
						 numFiles);
}

//------------------------------------------------------------------------------
const File& Phil::getCurrentDirectory() const
{
	static File retval;

	retval = currentDirectory;

	return retval;
}

//------------------------------------------------------------------------------
void Phil::setCurrentDirectory(const File& dir, bool passOnChangeMessage)
{
	if(dir != currentDirectory)
	{
		selectedFiles.clear();
		currentDirectory = dir;
		resized(); //So we see all the directory's contents.
		repaint();
		if(passOnChangeMessage)
			sendChangeMessage(this);
	}
}

//------------------------------------------------------------------------------
const StringArray& Phil::getCurrentlyHighlightedFiles() const
{
	static StringArray retval;

	retval.clear();

	for(int i=0;i<selectedFiles.size();++i)
		retval.add(selectedFiles[i]->getFullPathName());

	return retval;
}

//------------------------------------------------------------------------------
void Phil::setMultipleFilesSelected(bool val)
{
	multipleFilesSelected = val;
}

//------------------------------------------------------------------------------
void Phil::fileSelected(long index)
{
	File *newFile;
	OwnedArray<File> children;

	//Directories.
	findChildFiles(children, true);
	if(index < children.size())
		newFile = new File(*children[index]);
	else
	{
		index -= children.size();
		children.clear();
		//Files.
		findChildFiles(children, false);
		if(index < children.size())
			newFile = new File(*children[index]);
		else
		{
			selectedFiles.clear();
			repaint();
			sendChangeMessage(this);
			return;
		}
	}

	selectedFiles.clear();
	selectedFiles.add(newFile);

	repaint();

	//Scroll view to display the new selection.
	Rectangle currentView;
	const Rectangle& selectedView = fileView->getSelectedBounds();

	currentView.setBounds(view->getViewPositionX(),
						  view->getViewPositionY(),
						  view->getViewWidth(),
						  view->getViewHeight());

	if(!currentView.intersects(selectedView))
	{
		int x, y;

		x = selectedView.getX();
		y = currentView.getY();

		if((selectedView.getWidth() != 0)&&(selectedView.getHeight() != 0))
			view->setViewPosition(x, y);
	}

	sendChangeMessage(this);
}

//------------------------------------------------------------------------------
void Phil::anotherFileSelected(long index)
{
	File *newFile;
	OwnedArray<File> children;

	if(!multipleFilesSelected)
		return;

	//Directories.
	findChildFiles(children, true);
	if(index < children.size())
		newFile = new File(*children[index]);
	else
	{
		index -= children.size();
		children.clear();
		//Files.
		findChildFiles(children, false);
		if(index < children.size())
			newFile = new File(*children[index]);
		else
			return;
	}

	selectedFiles.add(newFile);

	repaint();

	sendChangeMessage(this);
}

//------------------------------------------------------------------------------
void Phil::selectAll(bool val)
{
	long i;
	OwnedArray<File> children;

	selectedFiles.clear();

	if(val)
	{
		//First the directories.
		findChildFiles(children, true);
		for(i=0;i<children.size();++i)
			selectedFiles.add(new File(*(children[i])));

		//Then the files.
		findChildFiles(children, false);
		for(i=0;i<children.size();++i)
			selectedFiles.add(new File(*(children[i])));
	}
	repaint();
	sendChangeMessage(this);
}

//------------------------------------------------------------------------------
void Phil::displayContextMenuNow(long file, int x, int y)
{
	long numFiles;
	PopupMenu menu;

	if(displayContextMenu)
	{
		numFiles = getNumberOfChildFiles(true);
		numFiles += getNumberOfChildFiles(false);
		if(file < numFiles)
		{
			fileSelected(file);
			//If it's a directory.
			if(file < getNumberOfChildFiles(true))
				menu.addItem(navReturn, T("Open"));
			//It's a file.
			else
			{
				if(!allowExternalPrograms)
					menu.addItem(navReturn, T("Run"), false);
				else
					menu.addItem(navReturn, T("Run"));
			}
			if(!allowExternalPrograms)
				menu.addItem(navSpace, T("Open with..."), false);
			else
				menu.addItem(navSpace, T("Open with..."));
			menu.addSeparator();
			menu.addItem(editCopy,
						 T("Copy"));
			menu.addItem(editCut,
						 T("Cut"));
			if(clipboardContents.size() > 0)
			{
				menu.addItem(editPaste,
							 T("Paste"),
							 true);
			}
			else
			{
				menu.addItem(editPaste,
							 T("Paste"),
							 false);
			}
			menu.addItem(editDelete, T("Delete"));
			menu.addSeparator();
			menu.addItem(navProperties, T("Properties"));
			menu.addSeparator();
			menu.addItem(editRename, T("Rename File/Directory..."));
			menu.addItem(editNewDir, T("New Directory..."));
		}
		else
		{
			menu.addItem(navReturn,
						 T("Open"),
						 false);
			menu.addItem(navSpace, T("Open with..."), false);
			menu.addSeparator();
			menu.addItem(editCopy,
						 T("Copy"),
						 false);
			menu.addItem(editCut,
						 T("Cut"),
						 false);
			if(clipboardContents.size() > 0)
			{
				menu.addItem(editPaste,
							 T("Paste"),
							 true);
			}
			else
			{
				menu.addItem(editPaste,
							 T("Paste"),
							 false);
			}
			menu.addItem(editDelete, T("Delete"), false);
			menu.addSeparator();
			menu.addItem(navProperties, T("Properties"), false);
			menu.addSeparator();
			menu.addItem(editRename, T("Rename File/Directory..."), false);
			menu.addItem(editNewDir, T("New Directory..."));
		}

		switch(menu.show())
		{
			case navReturn:
				enterObject();
				break;
			case navSpace:
				openWith();
				break;
			case editCopy:
				copyObject();
				break;
			case editCut:
				cutObject();
				break;
			case editPaste:
				pasteObject();
				break;
			case editDelete:
				deleteObject();
				break;
			case editRename:
				renameObject();
				break;
			case editNewDir:
				createNewDirectory();
				break;
			case navProperties:
				displayProperties();
				break;
		}
	}
}

//------------------------------------------------------------------------------
void Phil::maskExtensions(const StringArray& wildcardPattern, bool displayAll)
{
	if(displayAll)
	{
		visibleExtensions.clear();
		currentVisibleExtensions = -1;
	}
	else
	{
		visibleExtensions = wildcardPattern;
		currentVisibleExtensions = 0;
	}
}

//------------------------------------------------------------------------------
void Phil::setVisibleExtension(long index)
{
	currentVisibleExtensions = index;
	repaint();
	resized();
}

//------------------------------------------------------------------------------
int Phil::findChildFiles(OwnedArray<File> &results,
						 const bool searchForDirectories)
{
	StringComparator<File *> comp;

	if(currentDirectory == File::nonexistent)
	{
		if(searchForDirectories)
		{
#ifdef WIN32
			File::findFileSystemRoots(results);
			results.sort(comp);
			return results.size();
#else
			OwnedArray<File> tempa;
			File::findFileSystemRoots(tempa);
			if(tempa.size() > 0)
				tempa[0]->findChildFiles(results, true, false);
			results.sort(comp);
			return results.size();
#endif
		}
		else
		{
#ifdef WIN32
			return 0;
#else
			OwnedArray<File> tempa;
			File::findFileSystemRoots(tempa);
			if(tempa.size() > 0)
				tempa[0]->findChildFiles(results, false, false);
			results.sort(comp);
			return results.size();
#endif
		}
	}
	else
	{
		if(searchForDirectories)
		{
			int retval = 0;
			retval = currentDirectory.findChildFiles(results,
													 true,
													 false);
			results.sort(comp);
			return retval;
		}
		else
		{
			long i;
			int retval = 0;
			String tempstr;
			OwnedArray<File> tempFiles;

			if(visibleExtensions.size() == 0)
			{
				retval = currentDirectory.findChildFiles(results,
													   	 false,
													   	 false);
				results.sort(comp);
				return retval;
			}
			else
			{
				if((currentVisibleExtensions >= 0) &&
				   (currentVisibleExtensions < visibleExtensions.size()))
				{
					tempstr = T("*.");
					tempstr << visibleExtensions[currentVisibleExtensions];
					retval = currentDirectory.findChildFiles(tempFiles,
															 false,
															 false,
															 tempstr);
					for(i=0;i<tempFiles.size();++i)
						results.add(new File(tempFiles[i]->getFullPathName()));
					results.sort(comp);
				}
				else
				{
					retval = currentDirectory.findChildFiles(tempFiles,
															  false,
															  false);
					tempFiles.sort(comp);
				}

				return retval;
			}
		}
	}
}

//------------------------------------------------------------------------------
int Phil::getNumberOfChildFiles(const bool searchForDirectories)
{
	if(currentDirectory == File::nonexistent)
	{
		if(searchForDirectories)
		{
#ifdef WIN32
			OwnedArray<File> tempRoots;

			File::findFileSystemRoots(tempRoots);

			return tempRoots.size();
#else
			File fil;
			OwnedArray<File> tempRoots;

			File::findFileSystemRoots(tempRoots);
			if(tempRoots.size() > 0)
			{
				fil = *(tempRoots[0]);
				tempRoots.clear();
				return fil.findChildFiles(tempRoots, true, false);
			}
			else
				return 0;
#endif
		}
		else
		{
#ifdef WIN32
			return 0;
#else
			File fil;
			OwnedArray<File> tempRoots;

			File::findFileSystemRoots(tempRoots);
			if(tempRoots.size() > 0)
			{
				fil = *(tempRoots[0]);
				tempRoots.clear();
				return fil.findChildFiles(tempRoots, false, false);
			}
			else
				return 0;
#endif
		}
	}
	else
	{
		if(searchForDirectories)
			return currentDirectory.getNumberOfChildFiles(true);
		else
		{
			int retval;
			String tempstr;

			if((currentVisibleExtensions >= 0) &&
			   (currentVisibleExtensions < visibleExtensions.size()))
			{
				tempstr = T("*.");
				tempstr << visibleExtensions[currentVisibleExtensions];
				retval = currentDirectory.getNumberOfChildFiles(false,
																 tempstr);
			}
			else
				retval = currentDirectory.getNumberOfChildFiles(false);

			return retval;
		}
	}
}

//------------------------------------------------------------------------------
void Phil::copyObject()
{
	const StringArray& selected = getCurrentlyHighlightedFiles();

	//Put all selected files/directories in the clipboard.
	setClipboardContents(selected);
	setIsCut(false);
}

//------------------------------------------------------------------------------
void Phil::cutObject()
{
	const StringArray& selected = getCurrentlyHighlightedFiles();

	//Put all selected files/directories in the clipboard.
	setClipboardContents(selected);
	setIsCut(true);
}

//------------------------------------------------------------------------------
void Phil::pasteObject()
{
	long i;
	String tempstr;
	const StringArray& clipboard = getClipboard();

	if(currentDirectory == File::nonexistent)
		return;

	if(getIsCut())
	{
		for(i=0;i<clipboard.size();++i)
		{
			File fil(clipboard[i]);
			tempstr = getCurrentDirectory().getFullPathName();
			tempstr << File::separator << fil.getFileName();
			fil.moveFileTo(File(tempstr));
			//**Need to implement a moveDirectoryTo method?**
		}
	}
	else
	{
		for(i=0;i<clipboard.size();++i)
		{
			File fil(clipboard[i]);
			tempstr = getCurrentDirectory().getFullPathName();
			tempstr << File::separator << fil.getFileName();
			if(fil.isDirectory())
				fil.copyDirectoryTo(File(tempstr));
			else
				fil.copyFileTo(File(tempstr));
		}
	}
	repaint();
}

//------------------------------------------------------------------------------
void Phil::selectAllObjects()
{
	long tempint;

	tempint = getNumberOfChildFiles(true);
	tempint += getNumberOfChildFiles(false);
	if(tempint > getCurrentlyHighlightedFiles().size())
		selectAll(true);
	else
		selectAll(false);
}

//------------------------------------------------------------------------------
void Phil::deleteObject()
{
	long i;
	const StringArray& selected = getCurrentlyHighlightedFiles();

	if(displayContextMenu)
	{
		//Note: I don't use AlertWindow::showOkCancelBox() here because I need
		//		to set it to be always on top for it to display properly in
		//		Twindy.
		AlertWindow delObject(T("Caution"),
							  T("Are you sure you want to delete?"),
							  AlertWindow::InfoIcon);
		delObject.addButton(T("no"), 0);
		delObject.addButton(T("yes"),
					  	    1,
					  	    KeyPress(KeyPress::returnKey,
							   	     ModifierKeys()));

		delObject.setAlwaysOnTop(true);
		if(delObject.runModalLoop() == 1)
		{
			for(i=0;i<selected.size();++i)
				File(const_cast<const String&>(selected[i])).deleteRecursively();
			repaint();
		}
	}
	//Not sure if this is the best idea, but since PhilChooser's modal, and I
	//still want to be able to delete files when choosing them...
	else
	{
		for(i=0;i<selected.size();++i)
			File(const_cast<const String&>(selected[i])).deleteRecursively();
		repaint();
	}
}

//------------------------------------------------------------------------------
void Phil::navToParent()
{
	File tempFile = currentDirectory.getParentDirectory();

	//If we've reached the point below the file system root.
	if(tempFile == currentDirectory)
	{
		OwnedArray<File> topDirs;
		File::findFileSystemRoots(topDirs);

//#ifdef WIN32
		setCurrentDirectory(File::nonexistent);
/*#else
		//Because on linux and OSX, there will only be one root: /
		setCurrentDirectory(*(topDirs[0]));
		AlertWindow::showMessageBox(AlertWindow::InfoIcon,
									T("thing"),
									topDirs[0]->getFullPathName());
#endif*/
	}
	else
		setCurrentDirectory(tempFile);
}

//------------------------------------------------------------------------------
void Phil::cursorUp(bool multiple)
{
	long i;
	long numFiles;
	long currentIndex = 0;
	OwnedArray<File> childDirs;
	OwnedArray<File> childFiles;
	const StringArray& selected = getCurrentlyHighlightedFiles();

	findChildFiles(childDirs, true);
	findChildFiles(childFiles, false);
	numFiles = childDirs.size() + childFiles.size();

	//None selected - select first object.
	if(getCurrentlyHighlightedFiles().size() < 1)
	{
		if(numFiles > 0)
			fileSelected(numFiles-1);
	}
	//Something already selected - select previous object
	else
	{
		if(File(selected[0]).isDirectory())
		{
			for(i=0;i<childDirs.size();++i)
			{
				if(childDirs[i]->getFullPathName() == selected[0])
				{
					currentIndex = i;
					break;
				}
			}
		}
		else
		{
			for(i=0;i<childFiles.size();++i)
			{
				if(childFiles[i]->getFullPathName() == selected[0])
				{
					currentIndex = i + childDirs.size();
					break;
				}
			}
		}
		/*if(multiple)
		{
			if(currentIndex > 0)
				anotherFileSelected(currentIndex-1);
			else
				anotherFileSelected(numFiles-1);
		}
		else*/
		{
			if(currentIndex > 0)
				fileSelected(currentIndex-1);
			else
				fileSelected(numFiles-1);
		}
		repaint();
	}
}

//------------------------------------------------------------------------------
void Phil::cursorDown(bool multiple)
{
	long i;
	long numFiles;
	long currentIndex = 0;
	OwnedArray<File> childDirs;
	OwnedArray<File> childFiles;
	const StringArray& selected = getCurrentlyHighlightedFiles();

	findChildFiles(childDirs, true);
	findChildFiles(childFiles, false);
	numFiles = childDirs.size() + childFiles.size();

	//None selected - select last object.
	if(getCurrentlyHighlightedFiles().size() < 1)
	{
		if(numFiles > 0)
			fileSelected(0);
	}
	//Something already selected - select next object
	else
	{
		if(File(selected[0]).isDirectory())
		{
			for(i=0;i<childDirs.size();++i)
			{
				if(childDirs[i]->getFullPathName() == selected[0])
				{
					currentIndex = i;
					break;
				}
			}
		}
		else
		{
			for(i=0;i<childFiles.size();++i)
			{
				if(childFiles[i]->getFullPathName() == selected[0])
				{
					currentIndex = i + childDirs.size();
					break;
				}
			}
		}
		/*if(multiple)
		{
			if(currentIndex < (numFiles-1))
				anotherFileSelected(currentIndex+1);
			else
				anotherFileSelected(0);
		}
		else*/
		{
			if(currentIndex < (numFiles-1))
				fileSelected(currentIndex+1);
			else
				fileSelected(0);
		}
		repaint();
	}
}

//------------------------------------------------------------------------------
void Phil::cursorLeft()
{
	
}

//------------------------------------------------------------------------------
void Phil::cursorRight()
{
	
}

//------------------------------------------------------------------------------
void Phil::enterObject()
{
	const StringArray& selected = getCurrentlyHighlightedFiles();

	if(selected.size() > 0)
	{
		File fil(selected[0]);

		if(fil.isDirectory())
			setCurrentDirectory(fil);
		else if(allowExternalPrograms)
		{
			//Try to run application.
			if(!fil.startAsProcess())
			{
				AlertWindow dlg(T("Error:"),
								T("Sorry, could not run this executable."),
								AlertWindow::InfoIcon);
				dlg.addButton(T("ok"),
					  		  1,
					  		  KeyPress(KeyPress::returnKey,
							   		   ModifierKeys()));

				dlg.setAlwaysOnTop(true);
				dlg.runModalLoop();
			}
		}
	}
}

//------------------------------------------------------------------------------
void Phil::openWith()
{
	if((selectedFiles.size() > 0) && (allowExternalPrograms))
	{
#ifdef WIN32
		PhilChooser choose(T("Open with..."), currentDirectory, T("*.exe;*"));
		choose.setColours(fileViewBackgroundColour,
						  fileViewBoundaryColour,
						  fileViewTextColour,
						  fileViewHighlightColour);
		if(choose.browseForFileToOpen())
		{
			choose.getResult().startAsProcess(selectedFiles[0]->getFullPathName());
		}
#else
		PhilChooser choose(T("Open with..."), currentDirectory, T("*"));
		choose.setColours(fileViewBackgroundColour,
						  fileViewBoundaryColour,
						  fileViewTextColour,
						  fileViewHighlightColour);
		if(choose.browseForFileToOpen(0))
		{
			choose.getResult().startAsProcess(selectedFiles[0]->getFullPathName());
		}
#endif
	}
}

//------------------------------------------------------------------------------
void Phil::displayProperties()
{
	if((displayContextMenu) && (selectedFiles.size() > 0))
	{
#ifndef PHIL_LIGHT
		FilePropertiesDialog dlg(*(selectedFiles[0]),
								 fileViewBackgroundColour);
#endif
	}
}

//------------------------------------------------------------------------------
void Phil::createNewDirectory()
{
	String tempstr2;
	String tempstr = currentDirectory.getFullPathName();

	AlertWindow dirName(T("Create New Directory..."),
						T("Please enter the name of the directory."),
						AlertWindow::InfoIcon);
	dirName.addTextEditor(T("dirName"), T("<directory name>"));
	dirName.addButton(T("cancel"), 0);
	dirName.addButton(T("ok"),
					  1,
					  KeyPress(KeyPress::returnKey,
							   ModifierKeys()));

	dirName.setAlwaysOnTop(true);
	if(dirName.runModalLoop() == 1)
		tempstr2 = dirName.getTextEditorContents(T("dirName"));
	else
		return;

	tempstr << File::separator << tempstr2;

	File newDir(tempstr);
	if(!newDir.createDirectory())
	{
		AlertWindow dlg(T("Error:"),
						T("Sorry, could not create directory..."),
						AlertWindow::InfoIcon);
		dlg.addButton(T("ok"),
					  1,
					  KeyPress(KeyPress::returnKey,
							   ModifierKeys()));

		dlg.setAlwaysOnTop(true);
		dlg.runModalLoop();
	}
	repaint();
	resized();
}

//------------------------------------------------------------------------------
void Phil::renameObject()
{
	String tempstr2;

	AlertWindow newName(T("Rename File/Directory..."),
						T("Please enter the new name of the file/directory."),
						AlertWindow::InfoIcon);
	if(selectedFiles.size() > 0)
		newName.addTextEditor(T("newName"), selectedFiles[0]->getFileName());
	else
		newName.addTextEditor(T("newName"), T("<new name>"));
	newName.addButton(T("cancel"), 0);
	newName.addButton(T("ok"),
					  1,
					  KeyPress(KeyPress::returnKey,
							   ModifierKeys()));

	newName.setAlwaysOnTop(true);
	if(newName.runModalLoop() == 1)
		tempstr2 = newName.getTextEditorContents(T("newName"));
	else
		return;

	if(selectedFiles.size() > 0)
	{
		String tempstr = selectedFiles[0]->getParentDirectory().getFullPathName();
		tempstr << File::separator << tempstr2;
		if(!selectedFiles[0]->moveFileTo(File(tempstr)))
		{
			AlertWindow dlg(T("Error:"),
							T("Sorry, could not rename this file/directory..."),
							AlertWindow::InfoIcon);
			dlg.addButton(T("ok"),
					  	  1,
					  	  KeyPress(KeyPress::returnKey,
							   	   ModifierKeys()));

			dlg.setAlwaysOnTop(true);
			dlg.runModalLoop();
		}
		repaint();
	}
}

//------------------------------------------------------------------------------
const StringArray& Phil::getClipboard() const
{
	static StringArray retval;
	
	retval = clipboardContents;

	return retval;
}

//------------------------------------------------------------------------------
void Phil::setClipboardContents(const StringArray& newContents)
{
	clipboardContents.clear();
	clipboardContents = newContents;
}

//------------------------------------------------------------------------------
void Phil::setIsCut(bool val)
{
	isCut = val;
}

//------------------------------------------------------------------------------
void Phil::setDisplayContextMenu(bool val)
{
	displayContextMenu = val;
}

//------------------------------------------------------------------------------
void Phil::setAllowExternalPrograms(bool val)
{
	allowExternalPrograms = val;
}

//------------------------------------------------------------------------------
void Phil::setAllowDragAndDrop(bool val)
{
	allowDragAndDrop = val;
}

//------------------------------------------------------------------------------
void Phil::setFileView(FileView *newView)
{
	removeChildComponent(fileView);
	delete fileView;

	fileView = newView;
	addAndMakeVisible(fileView);
}

//------------------------------------------------------------------------------
void Phil::setToolbarComponent(Component *component)
{
	if(toolbarComponent)
	{
		removeChildComponent(toolbarComponent);
		delete toolbarComponent;
		toolbarComponent = 0;
	}

	toolbarComponent = component;
	addAndMakeVisible(toolbarComponent);

	repaint();
}

//------------------------------------------------------------------------------
void Phil::setToolbarHeight(int val)
{
	toolbarHeight = val;
	repaint();
}

//------------------------------------------------------------------------------
void Phil::setSidebarComponent(Component *component)
{
	if(sidebarComponent)
	{
		removeChildComponent(sidebarComponent);
		delete sidebarComponent;
		sidebarComponent = 0;
	}

	sidebarComponent = component;
	addAndMakeVisible(sidebarComponent);

	repaint();
}

//------------------------------------------------------------------------------
void Phil::setSidebarWidth(int val)
{
	sidebarWidth = val;
	repaint();
}

//------------------------------------------------------------------------------
void Phil::setStatusbarComponent(Component *component)
{
	if(statusbarComponent)
	{
		removeChildComponent(statusbarComponent);
		delete statusbarComponent;
		statusbarComponent = 0;
	}

	statusbarComponent = component;
	addAndMakeVisible(statusbarComponent);

	repaint();
}

//------------------------------------------------------------------------------
void Phil::setStatusbarHeight(int val)
{
	statusbarHeight = val;
	repaint();
}

//------------------------------------------------------------------------------
const Colour& Phil::getBackgroundColour() const
{
	static Colour retval;

	retval = fileViewBackgroundColour;

	return retval;
}

//------------------------------------------------------------------------------
const Colour& Phil::getBoundaryColour() const
{
	static Colour retval;

	retval = fileViewBoundaryColour;

	return retval;
}

//------------------------------------------------------------------------------
const Colour& Phil::getHighlightColour() const
{
	static Colour retval;

	retval = fileViewHighlightColour;

	return retval;
}

//------------------------------------------------------------------------------
const Colour& Phil::getTextColour() const
{
	static Colour retval;

	retval = fileViewTextColour;

	return retval;
}

//------------------------------------------------------------------------------
void Phil::setColours(const Colour& background,
					  const Colour& boundary,
					  const Colour& text,
					  const Colour& highlight)
{
	fileViewBackgroundColour = background;
	fileViewBoundaryColour = boundary;
	fileViewTextColour = text;
	fileViewHighlightColour = highlight;

	repaint();
}

//------------------------------------------------------------------------------
/*void Phil::shortcutCallback(const KeyPressCallbackFunctionInfo &keyPressInfo)
{
	Phil *phil;
	FileView *fView;

	if(keyPressInfo.componentWhereKeyWasPressed->getName() == T("Phil"))
		phil = static_cast<Phil *>(keyPressInfo.componentWhereKeyWasPressed);
	else if(keyPressInfo.componentWhereKeyWasPressed->getName() == T("FileView"))
	{
		fView = static_cast<FileView *>(keyPressInfo.componentWhereKeyWasPressed);
		phil = fView->getPhil();
	}
	else
		phil = 0;

	if(phil)
	{
		switch(keyPressInfo.commandUID)
		{
			case editCopy:
				phil->copyObject();
				break;
			case editCut:
				phil->cutObject();
				break;
			case editPaste:
				phil->pasteObject();
				break;
			case editSelect:
				phil->selectAllObjects();
				break;
			case editDeselect:
				phil->selectAll(false);
				break;
			case editDelete:
				phil->deleteObject();
				break;
			case editRename:
				phil->renameObject();
				break;
			case editNewDir:
				phil->createNewDirectory();
				break;
			case navParent:
				phil->navToParent();
				break;
			case navUp:
				phil->cursorUp(false);
				break;
			case navSelectUp:
				phil->cursorUp(true);
				break;
			case navDown:
				phil->cursorDown(false);
				break;
			case navSelectDown:
				phil->cursorDown(true);
				break;
			case navLeft:
				phil->cursorLeft();
				break;
			case navRight:
				phil->cursorRight();
				break;
			case navReturn:
				phil->enterObject();
				break;
			case navSpace:
				phil->openWith();
				break;
			case navProperties:
				phil->displayProperties();
				break;
		}
	}
}*/

//------------------------------------------------------------------------------
void Phil::getAllCommands(Array<CommandID>& commands)
{
	int i;

	for(i=editCopy;i<=navProperties;++i)
		commands.add(i);
}

//------------------------------------------------------------------------------
void Phil::getCommandInfo(const CommandID commandID,
						  ApplicationCommandInfo& result)
{
	switch(commandID)
	{
		case editCopy:
			result.setInfo(T("Copy"),
						   T("Copy a file/dir to the clipboard."),
						   T("Edit"),
						   0);
			result.addDefaultKeypress(T('c'), ModifierKeys::commandModifier);
			break;
		case editCut:
			result.setInfo(T("Cut"),
						   T("Cut a file/dir to the clipboard."),
						   T("Edit"),
						   0);
			result.addDefaultKeypress(T('x'), ModifierKeys::commandModifier);
			break;
		case editPaste:
			result.setInfo(T("Paste"),
						   T("Paste a file/dir to the current dir."),
						   T("Edit"),
						   0);
			result.addDefaultKeypress(T('v'), ModifierKeys::commandModifier);
			break;
		case editSelect:
			result.setInfo(T("Select All"),
						   T("Select/deselect all files."),
						   T("Edit"),
						   0);
			result.addDefaultKeypress(T('a'), ModifierKeys::commandModifier);
			break;
		case editDeselect:
			result.setInfo(T("Deselect All"),
						   T("Deselect all files."),
						   T("Edit"),
						   0);
			result.addDefaultKeypress(T('d'), ModifierKeys::commandModifier);
			break;
		case editDelete:
			result.setInfo(T("Delete"),
						   T("Delete the selected files."),
						   T("Edit"),
						   0);
			result.addDefaultKeypress(KeyPress::deleteKey, ModifierKeys());
			break;
		case editRename:
			result.setInfo(T("Rename"),
						   T("Rename the selected file/dir."),
						   T("Edit"),
						   0);
			result.addDefaultKeypress(T('r'), ModifierKeys::commandModifier);
			break;
		case editNewDir:
			result.setInfo(T("New dir"),
						   T("Create a new directory."),
						   T("Edit"),
						   0);
			result.addDefaultKeypress(T('n'), ModifierKeys::commandModifier);
			break;
		case navParent:
			result.setInfo(T("Parent dir"),
						   T("Navigate to the parent directory."),
						   T("Navigate"),
						   0);
			result.addDefaultKeypress(KeyPress::upKey, ModifierKeys::commandModifier);
			break;
		case navUp:
			result.setInfo(T("Nav up"),
						   T("Select the file/dir above this one."),
						   T("Navigate"),
						   0);
			result.addDefaultKeypress(KeyPress::upKey, ModifierKeys());
			break;
		case navSelectUp:
			result.setInfo(T("Nav select up"),
						   T("Select the file/dir above this one."),
						   T("Navigate"),
						   0);
			result.addDefaultKeypress(KeyPress::upKey, ModifierKeys::shiftModifier);
			break;
		case navDown:
			result.setInfo(T("Nav down"),
						   T("Select the file/dir below this one."),
						   T("Navigate"),
						   0);
			result.addDefaultKeypress(KeyPress::downKey, ModifierKeys());
			break;
		case navSelectDown:
			result.setInfo(T("Nav select down"),
						   T("Select the file/dir below this one."),
						   T("Navigate"),
						   0);
			result.addDefaultKeypress(KeyPress::downKey, ModifierKeys::shiftModifier);
			break;
		case navLeft:
			result.setInfo(T("Nav left"),
						   T("Select the file/dir next to this one."),
						   T("Navigate"),
						   0);
			result.addDefaultKeypress(KeyPress::leftKey, ModifierKeys());
			break;
		case navRight:
			result.setInfo(T("Nav right"),
						   T("Select the file/dir next to this one."),
						   T("Navigate"),
						   0);
			result.addDefaultKeypress(KeyPress::rightKey, ModifierKeys());
			break;
		case navReturn:
			result.setInfo(T("Nav return"),
						   T("Enter/run the currently-selected directory/file."),
						   T("Navigate"),
						   0);
			result.addDefaultKeypress(KeyPress::returnKey, ModifierKeys());
			break;
		case navSpace:
			result.setInfo(T("Nav space"),
						   T("Open the selected file/directory with a particular program."),
						   T("Navigate"),
						   0);
			result.addDefaultKeypress(KeyPress::spaceKey, ModifierKeys());
			break;
		case navProperties:
			result.setInfo(T("Nav properties"),
						   T("Display properties for the selected file/directory."),
						   T("Navigate"),
						   0);
			result.addDefaultKeypress(T('p'), ModifierKeys::commandModifier);
			break;
	}
}

//------------------------------------------------------------------------------
bool Phil::perform(const InvocationInfo& info)
{
	bool retval = false;

	switch(info.commandID)
	{
		case editCopy:
			copyObject();
			retval = true;
			break;
		case editCut:
			cutObject();
			retval = true;
			break;
		case editPaste:
			pasteObject();
			retval = true;
			break;
		case editSelect:
			selectAllObjects();
			retval = true;
			break;
		case editDeselect:
			selectAll(false);
			retval = true;
			break;
		case editDelete:
			deleteObject();
			retval = true;
			break;
		case editRename:
			renameObject();
			retval = true;
			break;
		case editNewDir:
			createNewDirectory();
			retval = true;
			break;
		case navParent:
			navToParent();
			retval = true;
			break;
		case navUp:
			cursorUp(false);
			retval = true;
			break;
		case navSelectUp:
			cursorUp(true);
			retval = true;
			break;
		case navDown:
			cursorDown(false);
			retval = true;
			break;
		case navSelectDown:
			cursorDown(true);
			retval = true;
			break;
		case navLeft:
			cursorLeft();
			retval = true;
			break;
		case navRight:
			cursorRight();
			retval = true;
			break;
		case navReturn:
			enterObject();
			retval = true;
			break;
		case navSpace:
			openWith();
			retval = true;
			break;
		case navProperties:
			displayProperties();
			retval = true;
			break;
	}

	return retval;
}

//------------------------------------------------------------------------------
void Phil::setupShortcuts()
{
	/*KeyPressCallbackFunction *callback = shortcutCallback;

	shortcuts.addCommand(editCopy,
						 T("Copy selected file."),
						 T("Edit"),
						 callback);
	shortcuts.addKeyPress(editCopy, KeyPress('c',
											 ModifierKeys::commandModifier));
	shortcuts.addCommand(editCut,
						 T("Cut selected file."),
						 T("Edit"),
						 callback);
	shortcuts.addKeyPress(editCut, KeyPress('x',
											ModifierKeys::commandModifier));
	shortcuts.addCommand(editPaste,
						 T("Paste selected file."),
						 T("Edit"),
						 callback);
	shortcuts.addKeyPress(editPaste, KeyPress('v',
											  ModifierKeys::commandModifier));
	shortcuts.addCommand(editSelect,
						 T("Select/deselect all files."),
						 T("Edit"),
						 callback);
	shortcuts.addKeyPress(editSelect, KeyPress('a',
											   ModifierKeys::commandModifier));
	shortcuts.addCommand(editDeselect,
						 T("Deselect all files."),
						 T("Edit"),
						 callback);
	shortcuts.addKeyPress(editDeselect, KeyPress('d',
											   ModifierKeys::commandModifier));
	shortcuts.addCommand(editDelete,
						 T("Delete selected file."),
						 T("Edit"),
						 callback);
	shortcuts.addKeyPress(editDelete, KeyPress(KeyPress::deleteKey,
											   ModifierKeys()));
	shortcuts.addCommand(editRename,
						 T("Rename selected file."),
						 T("Edit"),
						 callback);
	shortcuts.addKeyPress(editRename, KeyPress('r',
											   ModifierKeys::commandModifier));
	shortcuts.addCommand(editNewDir,
						 T("Create new directory."),
						 T("Edit"),
						 callback);
	shortcuts.addKeyPress(editNewDir, KeyPress('n',
											   ModifierKeys::commandModifier));
	shortcuts.addCommand(navParent,
						 T("Move to parent directory."),
						 T("Navigation"),
						 callback);
	shortcuts.addKeyPress(navParent, KeyPress(KeyPress::upKey,
											  ModifierKeys::commandModifier));
	shortcuts.addCommand(navUp,
						 T("Select file above this one."),
						 T("Navigation"),
						 callback);
	shortcuts.addKeyPress(navUp, KeyPress(KeyPress::upKey,
										  ModifierKeys()));
	shortcuts.addCommand(navSelectUp,
						 T("Select file above this one."),
						 T("Navigation"),
						 callback);
	shortcuts.addKeyPress(navSelectUp, KeyPress(KeyPress::upKey,
												ModifierKeys::shiftModifier));
	shortcuts.addCommand(navDown,
						 T("Select file below this one."),
						 T("Navigation"),
						 callback);
	shortcuts.addKeyPress(navDown, KeyPress(KeyPress::downKey,
										    ModifierKeys()));
	shortcuts.addCommand(navSelectDown,
						 T("Select file above this one."),
						 T("Navigation"),
						 callback);
	shortcuts.addKeyPress(navSelectDown, KeyPress(KeyPress::downKey,
												  ModifierKeys::shiftModifier));
	shortcuts.addCommand(navLeft,
						 T("Select file to the left of this one."),
						 T("Navigation"),
						 callback);
	shortcuts.addKeyPress(navLeft, KeyPress(KeyPress::leftKey,
										    ModifierKeys()));
	shortcuts.addCommand(navRight,
						 T("Select file to the right of this one."),
						 T("Navigation"),
						 callback);
	shortcuts.addKeyPress(navRight, KeyPress(KeyPress::rightKey,
										     ModifierKeys()));
	shortcuts.addCommand(navReturn,
						 T("Enter selected directory."),
						 T("Navigation"),
						 callback);
	shortcuts.addKeyPress(navReturn, KeyPress(KeyPress::returnKey,
										      ModifierKeys()));
	shortcuts.addCommand(navSpace,
						 T("Open object with..."),
						 T("Navigation"),
						 callback);
	shortcuts.addKeyPress(navSpace, KeyPress(KeyPress::spaceKey,
										     ModifierKeys()));
	shortcuts.addCommand(navProperties,
						 T("View file properties."),
						 T("Navigation"),
						 callback);
	shortcuts.addKeyPress(navProperties, KeyPress('p',
												  ModifierKeys::commandModifier));

	setWantsKeyboardFocus(true);
	addKeyListener(&shortcuts);
	fileView->addKeyListener(&shortcuts);*/
}

//#undef PHIL_LIGHT
